"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Uri = android.net.Uri;
var parameterBuilder_1 = require("./parameterBuilder");
var userInfo_1 = require("../../common/userInfo");
var databaseUser_1 = require("../../common/databaseUser");
var credentials_1 = require("../../common/credentials");
var requestFactory_1 = require("../request/internal/requestFactory");
var authenticationErrorBuilder_1 = require("../request/internal/authenticationErrorBuilder");
var databaseConnectionRequest_1 = require("./request/databaseConnectionRequest");
var profileRequest_1 = require("./request/profileRequest");
var signUpRequest_1 = require("./request/signUpRequest");
var tokenRequest_1 = require("./request/tokenRequest");
var AuthenticationAPIClient = (function () {
    function AuthenticationAPIClient(auth0, factory) {
        if (factory === void 0) { factory = new requestFactory_1.RequestFactory(); }
        this.auth0 = auth0;
        this.factory = factory;
        this.authErrorBuilder = new authenticationErrorBuilder_1.AuthenticationErrorBuilder();
        var telemetry = auth0.getTelemetry();
        if (telemetry != null) {
            factory.setClientInfo(telemetry.getValue());
        }
    }
    AuthenticationAPIClient.prototype.getClientId = function () {
        return this.auth0.getClientId();
    };
    AuthenticationAPIClient.prototype.getBaseURL = function () {
        return this.auth0.getDomainUrl();
    };
    AuthenticationAPIClient.prototype.setUserAgent = function (userAgent) {
        this.factory.setUserAgent(userAgent);
    };
    AuthenticationAPIClient.prototype.login = function (usernameOrEmail, password, realmOrConnection) {
        var builder = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.USERNAME_KEY, usernameOrEmail)
            .setParameter(AuthenticationAPIClient.PASSWORD_KEY, password);
        if (realmOrConnection != null) {
            builder.setGrantType(parameterBuilder_1.ParameterBuilder.GRANT_TYPE_PASSWORD_REALM).setRealm(realmOrConnection);
        }
        else {
            builder.setGrantType(parameterBuilder_1.ParameterBuilder.GRANT_TYPE_PASSWORD);
        }
        var parameters = builder.asDictionary();
        return this.loginWithToken(parameters);
    };
    AuthenticationAPIClient.prototype.loginWithOTP = function (mfaToken, otp) {
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setGrantType(parameterBuilder_1.ParameterBuilder.GRANT_TYPE_MFA_OTP)
            .setParameter(AuthenticationAPIClient.MFA_TOKEN_KEY, mfaToken)
            .setParameter(AuthenticationAPIClient.ONE_TIME_PASSWORD_KEY, otp)
            .asDictionary();
        return this.loginWithToken(parameters);
    };
    AuthenticationAPIClient.prototype.userInfo = function (accessToken) {
        return this.profileRequest()
            .addHeader(AuthenticationAPIClient.HEADER_AUTHORIZATION, "Bearer " + accessToken);
    };
    AuthenticationAPIClient.prototype.createUser = function (email, password, username, connection) {
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.DB_CONNECTIONS_PATH)
            .appendPath(AuthenticationAPIClient.SIGN_UP_PATH)
            .build();
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.USERNAME_KEY, username)
            .setParameter(AuthenticationAPIClient.EMAIL_KEY, email)
            .setParameter(AuthenticationAPIClient.PASSWORD_KEY, password)
            .setConnection(connection)
            .setClientId(this.getClientId())
            .asDictionary();
        var request = this.factory.POST(url, databaseUser_1.DatabaseUser, this.authErrorBuilder)
            .addParameters(parameters);
        return new databaseConnectionRequest_1.DatabaseConnectionRequest(request);
    };
    AuthenticationAPIClient.prototype.signUp = function (email, password, username, connection) {
        var createUserRequest = this.createUser(email, password, username, connection);
        var authenticationRequest = this.login(email, password, connection);
        return new signUpRequest_1.SignUpRequest(createUserRequest, authenticationRequest);
    };
    AuthenticationAPIClient.prototype.resetPassword = function (email, connection) {
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.DB_CONNECTIONS_PATH)
            .appendPath(AuthenticationAPIClient.CHANGE_PASSWORD_PATH)
            .build();
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.EMAIL_KEY, email)
            .setClientId(this.getClientId())
            .setConnection(connection)
            .asDictionary();
        var request = this.factory.POST(url, this.authErrorBuilder)
            .addParameters(parameters);
        return new databaseConnectionRequest_1.DatabaseConnectionRequest(request);
    };
    AuthenticationAPIClient.prototype.revokeToken = function (refreshToken) {
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setParameter(AuthenticationAPIClient.TOKEN_KEY, refreshToken)
            .asDictionary();
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.REVOKE_PATH)
            .build();
        return this.factory.POST(url, this.authErrorBuilder)
            .addParameters(parameters);
    };
    AuthenticationAPIClient.prototype.renewAuth = function (refreshToken) {
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setRefreshToken(refreshToken)
            .setGrantType(parameterBuilder_1.ParameterBuilder.GRANT_TYPE_REFRESH_TOKEN)
            .asDictionary();
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        return this.factory.POST(url, credentials_1.Credentials, this.authErrorBuilder)
            .addParameters(parameters);
    };
    AuthenticationAPIClient.prototype.getProfileAfter = function (authenticationRequest) {
        var profileRequest = this.profileRequest();
        return new profileRequest_1.ProfileRequest(authenticationRequest, profileRequest);
    };
    AuthenticationAPIClient.prototype.token = function (authorizationCode, redirectUri) {
        var parameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setGrantType(parameterBuilder_1.ParameterBuilder.GRANT_TYPE_AUTHORIZATION_CODE)
            .setParameter(AuthenticationAPIClient.OAUTH_CODE_KEY, authorizationCode)
            .setParameter(AuthenticationAPIClient.REDIRECT_URI_KEY, redirectUri)
            .asDictionary();
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        var request = this.factory.POST(url, credentials_1.Credentials, this.authErrorBuilder);
        request.addParameters(parameters);
        return new tokenRequest_1.TokenRequest(request);
    };
    AuthenticationAPIClient.prototype.loginWithToken = function (parameters) {
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        var requestParameters = parameterBuilder_1.ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .addAll(parameters)
            .asDictionary();
        return this.factory.authenticationPOST(url)
            .addAuthenticationParameters(requestParameters);
    };
    AuthenticationAPIClient.prototype.profileRequest = function () {
        var url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.USER_INFO_PATH)
            .build();
        return this.factory.GET(url, userInfo_1.UserInfo, this.authErrorBuilder);
    };
    AuthenticationAPIClient.USERNAME_KEY = "username";
    AuthenticationAPIClient.PASSWORD_KEY = "password";
    AuthenticationAPIClient.EMAIL_KEY = "email";
    AuthenticationAPIClient.OAUTH_CODE_KEY = "code";
    AuthenticationAPIClient.REDIRECT_URI_KEY = "redirect_uri";
    AuthenticationAPIClient.TOKEN_KEY = "token";
    AuthenticationAPIClient.MFA_TOKEN_KEY = "mfa_token";
    AuthenticationAPIClient.ONE_TIME_PASSWORD_KEY = "otp";
    AuthenticationAPIClient.SIGN_UP_PATH = "signup";
    AuthenticationAPIClient.DB_CONNECTIONS_PATH = "dbconnections";
    AuthenticationAPIClient.CHANGE_PASSWORD_PATH = "change_password";
    AuthenticationAPIClient.OAUTH_PATH = "oauth";
    AuthenticationAPIClient.TOKEN_PATH = "token";
    AuthenticationAPIClient.USER_INFO_PATH = "userinfo";
    AuthenticationAPIClient.REVOKE_PATH = "revoke";
    AuthenticationAPIClient.HEADER_AUTHORIZATION = "Authorization";
    return AuthenticationAPIClient;
}());
exports.AuthenticationAPIClient = AuthenticationAPIClient;
//# sourceMappingURL=authenticationAPIClient.js.map