"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TextUtils = android.text.TextUtils;
var Log = android.util.Log;
var auth0Exception_1 = require("../auth0Exception");
var passwordStrengthErrorParser_1 = require("./passwordStrengthErrorParser");
var AuthenticationException = (function (_super) {
    __extends(AuthenticationException, _super);
    function AuthenticationException(infoOrString, statusCode) {
        var _newTarget = this.constructor;
        if (infoOrString === void 0) { infoOrString = undefined; }
        if (statusCode === void 0) { statusCode = 0; }
        var _this = _super.call(this, AuthenticationException.DEFAULT_MESSAGE) || this;
        Object.setPrototypeOf(_this, _newTarget.prototype);
        if (typeof infoOrString === 'object') {
            _this.values = __assign({}, infoOrString);
            var codeValue = (_this.values[AuthenticationException.ERROR_KEY] != null
                ? _this.values[AuthenticationException.ERROR_KEY]
                : _this.values[AuthenticationException.CODE_KEY]);
            _this.code = codeValue != null ? codeValue : AuthenticationException.UNKNOWN_ERROR;
            if (_this.values[AuthenticationException.DESCRIPTION_KEY] == null) {
                _this.description = _this.values[AuthenticationException.ERROR_DESCRIPTION_KEY];
                _this.warnIfOIDCError();
                return _this;
            }
            var description = _this.values[AuthenticationException.DESCRIPTION_KEY];
            if (typeof description === 'string') {
                _this.description = description;
            }
            else if (_this.isPasswordNotStrongEnough()) {
                var pwStrengthParser = new passwordStrengthErrorParser_1.PasswordStrengthErrorParser(description);
                _this.description = pwStrengthParser.getDescription();
            }
        }
        else {
            _this.code = infoOrString != null ? AuthenticationException.NON_JSON_ERROR : AuthenticationException.EMPTY_BODY_ERROR;
            _this.description = infoOrString != null ? infoOrString : AuthenticationException.EMPTY_RESPONSE_BODY_DESCRIPTION;
            _this.statusCode = statusCode;
        }
        return _this;
    }
    AuthenticationException.prototype.warnIfOIDCError = function () {
        if ("invalid_request" === this.getCode() && (AuthenticationException.ERROR_OIDC_ACCESS_TOKEN === this.getDescription() || AuthenticationException.ERROR_OIDC_RO === this.getDescription())) {
            Log.w('AuthenticationAPIClient', "Your Auth0 Application is configured as 'OIDC Conformant' but this instance it's not. To authenticate you will need to enable the flag by calling Auth0#setOIDCConformant(true) on the Auth0 instance you used in the setup.");
        }
    };
    AuthenticationException.prototype.getCode = function () {
        return this.code != null ? this.code : AuthenticationException.UNKNOWN_ERROR;
    };
    AuthenticationException.prototype.getStatusCode = function () {
        return this.statusCode;
    };
    AuthenticationException.prototype.getDescription = function () {
        if (!TextUtils.isEmpty(this.description)) {
            return this.description;
        }
        if (AuthenticationException.UNKNOWN_ERROR === this.getCode()) {
            return "Received error with code " + this.getCode();
        }
        return "Failed with unknown error";
    };
    AuthenticationException.prototype.getValue = function (key) {
        if (this.values == null) {
            return null;
        }
        return this.values[key];
    };
    AuthenticationException.prototype.isBrowserAppNotAvailable = function () {
        return "a0.browser_not_available" === this.code;
    };
    AuthenticationException.prototype.isInvalidAuthorizeURL = function () {
        return "a0.invalid_authorize_url" === this.code;
    };
    AuthenticationException.prototype.isInvalidConfiguration = function () {
        return "a0.invalid_configuration" === this.code;
    };
    AuthenticationException.prototype.isMultifactorRequired = function () {
        return "mfa_required" === this.code || "a0.mfa_required" === this.code;
    };
    AuthenticationException.prototype.isMultifactorEnrollRequired = function () {
        return "a0.mfa_registration_required" === this.code || "unsupported_challenge_type" === this.code;
    };
    AuthenticationException.prototype.isMultifactorTokenInvalid = function () {
        return "expired_token" === this.code && "mfa_token is expired" === this.description ||
            "invalid_grant" === this.code && "Malformed mfa_token" === this.description;
    };
    AuthenticationException.prototype.isMultifactorCodeInvalid = function () {
        return "a0.mfa_invalid_code" === this.code || "invalid_grant" === this.code && "Invalid otp_code." === this.description;
    };
    AuthenticationException.prototype.isPasswordNotStrongEnough = function () {
        return "invalid_password" === this.code && "PasswordStrengthError" === this.values[AuthenticationException.NAME_KEY];
    };
    AuthenticationException.prototype.isPasswordAlreadyUsed = function () {
        return "invalid_password" === this.code && "PasswordHistoryError" === this.values[AuthenticationException.NAME_KEY];
    };
    AuthenticationException.prototype.isPasswordLeaked = function () {
        return "password_leaked" === this.code;
    };
    AuthenticationException.prototype.isRuleError = function () {
        return "unauthorized" === this.code;
    };
    AuthenticationException.prototype.isInvalidCredentials = function () {
        return "invalid_user_password" === this.code || "invalid_grant" === this.code && "Wrong email or password." === this.description;
    };
    AuthenticationException.prototype.isAccessDenied = function () {
        return "access_denied" === this.code;
    };
    AuthenticationException.prototype.isLoginRequired = function () {
        return "login_required" === this.code;
    };
    AuthenticationException.ERROR_KEY = "error";
    AuthenticationException.CODE_KEY = "code";
    AuthenticationException.DESCRIPTION_KEY = "description";
    AuthenticationException.ERROR_DESCRIPTION_KEY = "error_description";
    AuthenticationException.NAME_KEY = "name";
    AuthenticationException.DEFAULT_MESSAGE = "An error occurred when trying to authenticate with the server.";
    AuthenticationException.ERROR_OIDC_ACCESS_TOKEN = "OIDC conformant clients cannot use /oauth/access_token";
    AuthenticationException.ERROR_OIDC_RO = "OIDC conformant clients cannot use /oauth/ro";
    return AuthenticationException;
}(auth0Exception_1.Auth0Exception));
exports.AuthenticationException = AuthenticationException;
//# sourceMappingURL=authenticationException.js.map