"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Intent = android.content.Intent;
var Uri = android.net.Uri;
var Log = android.util.Log;
var oauthManager_1 = require("./oauthManager");
var responseType_1 = require("./responseType");
var flagChecker_1 = require("./flagChecker");
var callbackHelper_1 = require("./callbackHelper");
var authenticationException_1 = require("../authentication/authenticationException");
var authorizeResult_1 = require("./authorizeResult");
var WebAuthProvider = (function () {
    function WebAuthProvider(account) {
        this.account = account;
        this.values = {};
        this.scheme = "https";
        this.withResponseType(responseType_1.ResponseType.CODE);
        this.withScope(WebAuthProvider.SCOPE_TYPE_OPENID);
    }
    WebAuthProvider.prototype.withState = function (state) {
        this.values[oauthManager_1.OAuthManager.KEY_STATE] = state;
        return this;
    };
    WebAuthProvider.prototype.withNonce = function (nonce) {
        this.values[oauthManager_1.OAuthManager.KEY_NONCE] = nonce;
        return this;
    };
    WebAuthProvider.prototype.withAudience = function (audience) {
        this.values[WebAuthProvider.KEY_AUDIENCE] = audience;
        return this;
    };
    WebAuthProvider.prototype.withScheme = function (scheme) {
        var lowerCase = scheme.toLowerCase();
        if (scheme !== lowerCase) {
            Log.w(WebAuthProvider.TAG, "Please provide the scheme in lowercase and make sure it's the same configured in the intent filter. Android expects the scheme in lowercase");
        }
        this.scheme = scheme;
        return this;
    };
    WebAuthProvider.prototype.withScope = function (scope) {
        this.values[WebAuthProvider.KEY_SCOPE] = scope;
        return this;
    };
    WebAuthProvider.prototype.withConnectionScope = function () {
        var connectionScope = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            connectionScope[_i] = arguments[_i];
        }
        var concatScope = connectionScope.map(function (x) { return x.trim(); }).join(' ');
        if (concatScope.length > 0) {
            this.values[WebAuthProvider.KEY_CONNECTION_SCOPE] = concatScope;
        }
        return this;
    };
    WebAuthProvider.prototype.withResponseType = function (type) {
        var sb = '';
        if (flagChecker_1.FlagChecker.hasFlag(type, responseType_1.ResponseType.CODE)) {
            sb += oauthManager_1.OAuthManager.RESPONSE_TYPE_CODE + " ";
        }
        if (flagChecker_1.FlagChecker.hasFlag(type, responseType_1.ResponseType.ID_TOKEN)) {
            sb += oauthManager_1.OAuthManager.RESPONSE_TYPE_ID_TOKEN + " ";
        }
        if (flagChecker_1.FlagChecker.hasFlag(type, responseType_1.ResponseType.TOKEN)) {
            sb += "" + WebAuthProvider.RESPONSE_TYPE_TOKEN;
        }
        this.values[oauthManager_1.OAuthManager.KEY_RESPONSE_TYPE] = sb.trim();
        return this;
    };
    WebAuthProvider.prototype.withParameters = function (parameters) {
        for (var key in parameters) {
            var value = parameters[key];
            if (value != null) {
                this.values[key] = value.toString();
            }
        }
        return this;
    };
    WebAuthProvider.prototype.withConnection = function (connectionName) {
        this.values[oauthManager_1.OAuthManager.KEY_CONNECTION] = connectionName;
        return this;
    };
    WebAuthProvider.prototype.withCustomTabsOptions = function (options) {
        this.ctOptions = options;
        return this;
    };
    WebAuthProvider.prototype.withPKCE = function (pkce) {
        this.pkce = pkce;
        return this;
    };
    WebAuthProvider.hasBrowserAppInstalled = function (packageManager) {
        var intent = new Intent(Intent.ACTION_VIEW, Uri.parse("https://auth0.com"));
        return intent.resolveActivity(packageManager) != null;
    };
    WebAuthProvider.prototype.start = function (activity, callback) {
        WebAuthProvider.managerInstance = null;
        if (this.account.getAuthorizeUrl() == null) {
            var ex = new authenticationException_1.AuthenticationException({
                code: "a0.invalid_authorize_url",
                description: "Auth0 authorize URL not properly set. This can be related to an invalid domain."
            }, 0);
            callback.onFailure(ex);
            return;
        }
        if (!WebAuthProvider.hasBrowserAppInstalled(activity.getPackageManager())) {
            var ex = new authenticationException_1.AuthenticationException({
                code: "a0.browser_not_available",
                description: "No Browser application installed to perform web authentication."
            }, 0);
            callback.onFailure(ex);
            return;
        }
        var manager = new oauthManager_1.OAuthManager(this.account, callback, this.values);
        manager.setCustomTabsOptions(this.ctOptions);
        manager.setPKCE(this.pkce);
        WebAuthProvider.managerInstance = manager;
        var redirectUri = callbackHelper_1.CallbackHelper.getCallbackUri(this.scheme, activity.getApplicationContext().getPackageName(), this.account.getDomainUrl());
        manager.startAuthorization(activity, redirectUri, 110);
    };
    WebAuthProvider.resume = function (intent) {
        if (WebAuthProvider.managerInstance == null) {
            Log.w(WebAuthProvider.TAG, "There is no previous instance of this provider.");
            return false;
        }
        var data = new authorizeResult_1.AuthorizeResult(intent);
        var success = WebAuthProvider.managerInstance.resumeAuthorization(data);
        if (success) {
            WebAuthProvider.managerInstance = null;
        }
        return success;
    };
    WebAuthProvider.getInstance = function () {
        return WebAuthProvider.managerInstance;
    };
    WebAuthProvider.TAG = 'WebAuthProvider';
    WebAuthProvider.KEY_AUDIENCE = "audience";
    WebAuthProvider.KEY_SCOPE = "scope";
    WebAuthProvider.KEY_CONNECTION_SCOPE = "connection_scope";
    WebAuthProvider.SCOPE_TYPE_OPENID = "openid";
    WebAuthProvider.RESPONSE_TYPE_TOKEN = "token";
    return WebAuthProvider;
}());
exports.WebAuthProvider = WebAuthProvider;
//# sourceMappingURL=webAuthProvider.js.map