"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = require("tns-core-modules/http/http");
var IOException = java.io.IOException;
var auth0Exception_1 = require("../../auth0Exception");
var parameterBuilder_1 = require("../../authentication/parameterBuilder");
var BaseRequest = (function () {
    function BaseRequest(url, adapter, errorBuilder, headers, parameterBuilder) {
        if (headers === void 0) { headers = {}; }
        if (parameterBuilder === void 0) { parameterBuilder = parameterBuilder_1.ParameterBuilder.newBuilder(); }
        this.url = url;
        this.adapter = adapter;
        this.headers = headers;
        this.builder = parameterBuilder;
        this.errorBuilder = errorBuilder;
    }
    BaseRequest.prototype.setCallback = function (callback) {
        this.callback = callback;
    };
    BaseRequest.prototype.postOnSuccess = function (payload) {
        this.callback.onSuccess(payload);
    };
    BaseRequest.prototype.postOnFailure = function (error) {
        this.callback.onFailure(error);
    };
    BaseRequest.prototype.newRequestOptions = function (method) {
        var options = {
            url: this.url.toString(),
            method: method,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        for (var entry in this.headers) {
            options.headers[entry] = this.headers[entry];
        }
        return options;
    };
    BaseRequest.prototype.getAdapter = function () {
        return this.adapter;
    };
    BaseRequest.prototype.getErrorBuilder = function () {
        return this.errorBuilder;
    };
    BaseRequest.prototype.getCallback = function () {
        return this.callback;
    };
    BaseRequest.prototype.buildBody = function () {
        var dictionary = this.builder.asDictionary();
        if (Object.keys(dictionary).length > 0) {
            return JSON.stringify(dictionary);
        }
        return null;
    };
    BaseRequest.prototype.parseUnsuccessfulResponse = function (response) {
        var stringPayload = undefined;
        try {
            stringPayload = response.content.toString(http_1.HttpResponseEncoding.UTF8);
            return this.errorBuilder.from(JSON.parse(stringPayload));
        }
        catch (e) {
            if (e instanceof SyntaxError) {
                return this.errorBuilder.from(stringPayload, response.statusCode);
            }
            else if (e instanceof IOException) {
                var auth0Exception = new auth0Exception_1.Auth0Exception("Error parsing the server response " + e.toString());
                return this.errorBuilder.from("Request to " + this.url.toString() + " failed", auth0Exception);
            }
            else {
                throw e;
            }
        }
    };
    BaseRequest.prototype.onFailure = function (request, e) {
        var exception = new auth0Exception_1.Auth0Exception("Failed to execute request to " + this.url.toString() + " - " + e.toString());
        this.postOnFailure(this.errorBuilder.from("Request failed", exception));
    };
    BaseRequest.prototype.addHeader = function (name, value) {
        this.headers[name] = value;
        return this;
    };
    BaseRequest.prototype.setBearer = function (jwt) {
        this.addHeader("Authorization", "Bearer " + jwt);
        return this;
    };
    BaseRequest.prototype.addParameters = function (parameters) {
        this.builder.addAll(parameters);
        return this;
    };
    BaseRequest.prototype.addParameter = function (name, value) {
        this.builder.setParameter(name, value);
        return this;
    };
    BaseRequest.prototype.start = function (callback) {
        var _this = this;
        this.setCallback(callback);
        var request = this.doBuildRequest();
        http_1.request(request)
            .then(function (response) {
            _this.onResponse(response);
        })
            .catch(function (error) {
            _this.onFailure(request, error);
        });
    };
    return BaseRequest;
}());
exports.BaseRequest = BaseRequest;
//# sourceMappingURL=baseRequest.js.map