"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var authentication_1 = require("./authentication");
var telemetry_1 = require("./telemetry");
var credentials_1 = require("../common/credentials");
var authenticationError_1 = require("./authenticationError");
var request_1 = require("./request");
var handlers_1 = require("./handlers");
var userInfo_1 = require("../common/userInfo");
var safariWebAuth_1 = require("./safariWebAuth");
var controllerModalPresenter_1 = require("./controllerModalPresenter");
var loggable_1 = require("./loggable");
var databaseUser_1 = require("../common/databaseUser");
var Auth0Authentication = (function (_super) {
    __extends(Auth0Authentication, _super);
    function Auth0Authentication(clientId, url, telemetry) {
        if (telemetry === void 0) { telemetry = new telemetry_1.Telemetry(); }
        var _this = _super.call(this) || this;
        _this.clientId = clientId;
        _this.url = url;
        _this.telemetry = telemetry;
        return _this;
    }
    Auth0Authentication.prototype.login = function (username, password, realm, audience, scope, parameters) {
        var resourceOwner = new NSURL({ string: "/oauth/token", relativeToURL: this.url });
        var payload = {
            "username": username,
            "password": password,
            "grant_type": "http://auth0.com/oauth/grant-type/password-realm",
            "client_id": this.clientId,
            "realm": realm
        };
        payload["audience"] = audience;
        payload["scope"] = scope;
        if (parameters != null) {
            for (var key in parameters) {
                payload[key] = parameters[key];
            }
        }
        return new request_1.Request(resourceOwner, "POST", handlers_1.authenticationObject(credentials_1.Credentials), payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.loginWithOTP = function (otp, mfaToken) {
        var url = new NSURL({ string: "/oauth/token", relativeToURL: this.url });
        var payload = {
            "otp": otp,
            "mfa_token": mfaToken,
            "grant_type": "http://auth0.com/oauth/grant-type/mfa-otp",
            "client_id": this.clientId
        };
        return new request_1.Request(url, "POST", handlers_1.authenticationObject(credentials_1.Credentials), payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.createUser = function (email, username, password, connection, userMetadata) {
        if (username === void 0) { username = undefined; }
        if (userMetadata === void 0) { userMetadata = undefined; }
        var payload = {
            "email": email,
            "password": password,
            "connection": connection,
            "client_id": this.clientId
        };
        payload["username"] = username;
        payload["user_metadata"] = userMetadata;
        var createUser = new NSURL({ string: "/dbconnections/signup", relativeToURL: this.url });
        return new request_1.Request(createUser, "POST", handlers_1.authenticationObject(databaseUser_1.DatabaseUser), payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.resetPassword = function (email, connection) {
        var payload = {
            "email": email,
            "connection": connection,
            "client_id": this.clientId
        };
        var resetPassword = new NSURL({ string: "/dbconnections/change_password", relativeToURL: this.url });
        return new request_1.Request(resetPassword, "POST", handlers_1.noBody, payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.userInfo = function (accessToken) {
        var userInfo = new NSURL({ string: "/userinfo", relativeToURL: this.url });
        return new request_1.Request(userInfo, "GET", handlers_1.authenticationObject(userInfo_1.UserInfo), {}, { "Authorization": "Bearer " + accessToken }, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.tokenExchangeWithParameters = function (parameters) {
        var payload = {
            "client_id": this.clientId
        };
        for (var key in parameters) {
            payload[key] = parameters[key];
        }
        var token = new NSURL({ string: "/oauth/token", relativeToURL: this.url });
        return new request_1.Request(token, "POST", handlers_1.authenticationObject(credentials_1.Credentials), payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.tokenExchangeWithCode = function (code, codeVerifier, redirectURI) {
        return this.tokenExchangeWithParameters({
            "code": code,
            "code_verifier": codeVerifier,
            "redirect_uri": redirectURI,
            "grant_type": "authorization_code"
        });
    };
    Auth0Authentication.prototype.renew = function (refreshToken) {
        var payload = {
            "refresh_token": refreshToken,
            "grant_type": "refresh_token",
            "client_id": this.clientId
        };
        var oauthToken = new NSURL({ string: "/oauth/token", relativeToURL: this.url });
        return new request_1.Request(oauthToken, "POST", handlers_1.authenticationObject(credentials_1.Credentials), payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.revoke = function (refreshToken) {
        var payload = {
            "token": refreshToken,
            "client_id": this.clientId
        };
        var oauthToken = new NSURL({ string: "/oauth/revoke", relativeToURL: this.url });
        return new request_1.Request(oauthToken, "POST", handlers_1.noBody, payload, {}, this.logger, this.telemetry, authenticationError_1.AuthenticationError);
    };
    Auth0Authentication.prototype.webAuth = function (connection) {
        var safari = safariWebAuth_1.SafariWebAuth.init(this.clientId, this.url, new controllerModalPresenter_1.ControllerModalPresenter(), this.telemetry);
        loggable_1.LoggableExtension.logging(safari, this.logger != null);
        return safari.setConnection(connection);
    };
    return Auth0Authentication;
}(authentication_1.Authentication));
exports.Auth0Authentication = Auth0Authentication;
//# sourceMappingURL=auth0Authentication.js.map