"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var credentials_1 = require("../common/credentials");
var responseType_1 = require("./responseType");
var utils_1 = require("./utils");
var webAuthError_1 = require("./webAuthError");
var authenticationError_1 = require("./authenticationError");
var ImplicitGrant = (function () {
    function ImplicitGrant(responseType, nonce) {
        if (responseType === void 0) { responseType = [responseType_1.ResponseType.token]; }
        if (nonce === void 0) { nonce = undefined; }
        this.responseType = responseType;
        if (nonce != null) {
            this.defaults = { "nonce": nonce };
        }
        else {
            this.defaults = {};
        }
    }
    ImplicitGrant.prototype.credentials = function (values, callback) {
        var valid = validate(this.responseType, values["id_token"], this.defaults["nonce"]);
        if (!valid) {
            return callback({
                failure: webAuthError_1.WebAuthError.invalidIdTokenNonce
            });
        }
        if (this.responseType.indexOf(responseType_1.ResponseType.token) > -1 && values["access_token"] == null) {
            return callback({
                failure: webAuthError_1.WebAuthError.missingAccessToken
            });
        }
        callback({
            success: credentials_1.Credentials.initWithJson(values)
        });
    };
    ImplicitGrant.prototype.values = function (components) {
        return utils_1.a0_fragmentValues(components);
    };
    return ImplicitGrant;
}());
exports.ImplicitGrant = ImplicitGrant;
var PKCE = (function () {
    function PKCE(authentication, redirectURL, verifier, challenge, method, responseType, nonce) {
        if (nonce === void 0) { nonce = undefined; }
        this.authentication = authentication;
        this.redirectURL = redirectURL;
        this.verifier = verifier;
        this.responseType = responseType;
        var newDefaults = {
            "code_challenge": challenge,
            "code_challenge_method": method
        };
        if (nonce != null) {
            newDefaults["nonce"] = nonce;
        }
        this.defaults = newDefaults;
    }
    PKCE.init = function (authentication, redirectURL, reponseType, nonce, generator) {
        if (reponseType === void 0) { reponseType = [responseType_1.ResponseType.code]; }
        if (nonce === void 0) { nonce = undefined; }
        if (generator === void 0) { generator = A0SHA256ChallengeGenerator.alloc().init(); }
        return new PKCE(authentication, redirectURL, generator.verifier, generator.challenge, generator.method, reponseType, nonce);
    };
    PKCE.prototype.credentials = function (values, callback) {
        var code = values["code"];
        if (code == null) {
            var _string = "No code found in parameters \(values)";
            return callback({
                failure: new authenticationError_1.AuthenticationError(_string)
            });
        }
        var valid = validate(this.responseType, values["id_token"], this.defaults["nonce"]);
        if (valid === false) {
            return callback({
                failure: webAuthError_1.WebAuthError.invalidIdTokenNonce
            });
        }
        var clientId = this.authentication.clientId;
        this.authentication
            .tokenExchangeWithCode(code, this.verifier, this.redirectURL.absoluteString)
            .start(function (result) {
            if (result.failure != null && result.failure.message === "Unauthorized") {
                var error = webAuthError_1.WebAuthError.pkceNotAllowed("Please go to 'https://manage.auth0.com/#/applications/" + clientId + "/settings' and make sure 'Client Type' is 'Native' to enable PKCE.");
                callback({
                    failure: error
                });
            }
            else {
                callback(result);
            }
        });
    };
    PKCE.prototype.values = function (components) {
        var items = utils_1.a0_fragmentValues(components);
        var values = utils_1.a0_queryValues(components);
        for (var key in values) {
            items[key] = values[key];
        }
        return items;
    };
    return PKCE;
}());
exports.PKCE = PKCE;
function validate(responseType, token, nonce) {
    var index = responseType.indexOf(responseType_1.ResponseType.idToken);
    if (index === -1) {
        return true;
    }
    if (nonce == null || token == null) {
        return false;
    }
    var claims = decode(token);
    var actualNonce = (claims != null) ? claims["nonce"] : undefined;
    return actualNonce === nonce;
}
function decode(jwt) {
    var parts = jwt.split('.');
    if (parts.length !== 3) {
        return undefined;
    }
    var base64 = parts[1]
        .replace('-', '+')
        .replace('_', '/');
    var length = NSString.stringWithString(base64).lengthOfBytesUsingEncoding(NSUTF8StringEncoding);
    var requiredLength = 4 * ceil(length / 4.0);
    var paddingLength = requiredLength - length;
    if (paddingLength > 0) {
        var padding = NSString.string().stringByPaddingToLengthWithStringStartingAtIndex(paddingLength, '=', 0);
        base64 += padding;
    }
    var bodyData = new NSData({ base64EncodedString: base64, options: 1 });
    if (bodyData == null) {
        return undefined;
    }
    var jsonString = String(NSString.alloc().initWithDataEncoding(bodyData, NSUTF8StringEncoding));
    return JSON.parse(jsonString);
}
//# sourceMappingURL=oauth2Grant.js.map