"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var platform_1 = require("tns-core-modules/platform/platform");
var controllerModalPresenter_1 = require("./controllerModalPresenter");
var responseType_1 = require("./responseType");
var telemetry_1 = require("./telemetry");
var transactionStore_1 = require("./transactionStore");
var webAuth_1 = require("./webAuth");
var utils_1 = require("./utils");
var webAuthError_1 = require("./webAuthError");
var safariAuthenticationSession_1 = require("./safariAuthenticationSession");
var safariSession_1 = require("./safariSession");
var oauth2Grant_1 = require("./oauth2Grant");
var safariAuthenticationSessionCallback_1 = require("./safariAuthenticationSessionCallback");
var silentSafariViewController_1 = require("./silentSafariViewController");
var auth0Authentication_1 = require("./auth0Authentication");
var SafariWebAuth = (function (_super) {
    __extends(SafariWebAuth, _super);
    function SafariWebAuth(clientId, url, presenter, storage, telemetry) {
        var _this = _super.call(this) || this;
        _this.parameters = {};
        _this.universalLink = false;
        _this.responseType = [responseType_1.ResponseType.code];
        _this.authenticationSession = true;
        _this.clientId = clientId;
        _this.url = url;
        _this.presenter = presenter;
        _this.storage = storage;
        _this.telemetry = telemetry;
        return _this;
    }
    SafariWebAuth.init = function (clientId, url, presenter, telemetry) {
        if (presenter === void 0) { presenter = new controllerModalPresenter_1.ControllerModalPresenter(); }
        if (telemetry === void 0) { telemetry = new telemetry_1.Telemetry(); }
        return new SafariWebAuth(clientId, url, presenter, transactionStore_1.TransactionStore.shared, telemetry);
    };
    SafariWebAuth.prototype.useUniversalLink = function () {
        this.universalLink = true;
        return this;
    };
    SafariWebAuth.prototype.setConnection = function (connection) {
        this.parameters["connection"] = connection;
        return this;
    };
    SafariWebAuth.prototype.setScope = function (scope) {
        this.parameters["scope"] = scope;
        return this;
    };
    SafariWebAuth.prototype.setConnectionScope = function (connectionScope) {
        this.parameters["connection_scope"] = connectionScope;
        return this;
    };
    SafariWebAuth.prototype.setState = function (state) {
        this.parameters["state"] = state;
        return this;
    };
    SafariWebAuth.prototype.setParameters = function (parameters) {
        for (var key in parameters) {
            this.parameters[key] = parameters[key];
        }
        return this;
    };
    SafariWebAuth.prototype.setResponseType = function (responseType) {
        this.responseType = responseType;
        return this;
    };
    SafariWebAuth.prototype.setNonce = function (nonce) {
        this.nonce = nonce;
        return this;
    };
    SafariWebAuth.prototype.usingImplicitGrant = function () {
        return this.setResponseType([responseType_1.ResponseType.token]);
    };
    SafariWebAuth.prototype.setAudience = function (audience) {
        this.parameters["audience"] = audience;
        return this;
    };
    SafariWebAuth.prototype.useLegacyAuthentication = function () {
        this.authenticationSession = false;
        return this;
    };
    SafariWebAuth.prototype.start = function (callback) {
        var redirectURL = this.redirectURL;
        if (redirectURL == null || redirectURL.absoluteString.startsWith(SafariWebAuth.NoBundleIdentifier)) {
            return callback({
                failure: webAuthError_1.WebAuthError.noBundleIdentifierFound
            });
        }
        if (this.responseType.indexOf(responseType_1.ResponseType.idToken) > -1) {
            if (this.nonce == null) {
                return callback({
                    failure: webAuthError_1.WebAuthError.noNonceProvided
                });
            }
        }
        var handler = this.handler(redirectURL);
        var state = this.parameters["state"] || generateDefaultState();
        var authorizeURL = this.buildAuthorizeURL(redirectURL, handler.defaults, state);
        if (Number(platform_1.device.osVersion.split('.')[0]) >= 11.0 && this.authenticationSession) {
            var session = new safariAuthenticationSession_1.SafariAuthenticationSession(authorizeURL, redirectURL, state, handler, callback, this.logger);
            if (this.logger != null) {
                this.logger.trace(authorizeURL, "SafariAuthenticationSession");
            }
            this.storage.store(session);
        }
        else {
            var _a = this.newSafari(authorizeURL, callback), controller = _a.controller, finish = _a.finish;
            var session = new safariSession_1.SafariSession(controller, redirectURL, state, handler, finish, this.logger);
            this.presenter.present(controller);
            if (this.logger != null) {
                this.logger.trace(authorizeURL, "Safari");
            }
            this.storage.store(session);
        }
    };
    SafariWebAuth.prototype.newSafari = function (authorizeURL, callback) {
        var controller = new SFSafariViewController({ URL: authorizeURL });
        var finish = function (result) {
            var presenting = (controller != null) ? controller.presentingViewController : undefined;
            if (presenting == null) {
                return callback({
                    failure: webAuthError_1.WebAuthError.cannotDismissWebAuthController
                });
            }
            if (result.failure != null && result.failure === webAuthError_1.WebAuthError.userCancelled) {
                utils_1.invokeOnRunLoop(function () {
                    callback(result);
                });
            }
            else {
                utils_1.invokeOnRunLoop(function () {
                    presenting.dismissViewControllerAnimatedCompletion(true, function () {
                        callback(result);
                    });
                });
            }
        };
        return { controller: controller, finish: finish };
    };
    SafariWebAuth.prototype.buildAuthorizeURL = function (redirectURL, defaults, state) {
        var authorize = new NSURL({ string: "/authorize", relativeToURL: this.url });
        var components = new NSURLComponents({ URL: authorize, resolvingAgainstBaseURL: true });
        var items = [];
        var entries = defaults;
        entries["client_id"] = this.clientId;
        entries["redirect_uri"] = redirectURL.absoluteString;
        entries["scope"] = "openid";
        entries["state"] = state;
        entries["response_type"] = this.responseType.map(function (x) { return x.label; }).join(" ");
        if (this.responseType.indexOf(responseType_1.ResponseType.idToken) > -1) {
            entries["nonce"] = this.nonce;
        }
        for (var key in this.parameters) {
            entries[key] = this.parameters[key];
        }
        for (var key in entries) {
            items.push(new NSURLQueryItem({ name: key, value: entries[key] }));
        }
        components.queryItems = utils_1.jsArrayToNSArray(this.telemetry.queryItemsWithTelemetry(items));
        return components.URL;
    };
    SafariWebAuth.prototype.handler = function (redirectURL) {
        if (this.responseType.indexOf(responseType_1.ResponseType.code) > -1) {
            var authentication = new auth0Authentication_1.Auth0Authentication(this.clientId, this.url, this.telemetry);
            authentication.logger = this.logger;
            return oauth2Grant_1.PKCE.init(authentication, redirectURL, this.responseType, this.nonce);
        }
        else {
            return new oauth2Grant_1.ImplicitGrant(this.responseType, this.nonce);
        }
    };
    Object.defineProperty(SafariWebAuth.prototype, "redirectURL", {
        get: function () {
            var bundleIdentifier = NSBundle.mainBundle.bundleIdentifier || SafariWebAuth.NoBundleIdentifier;
            var components = new NSURLComponents({ URL: this.url, resolvingAgainstBaseURL: true });
            if (components != null && components.URL != null) {
                components.scheme = this.universalLink ? "https" : bundleIdentifier;
                return components.URL
                    .URLByAppendingPathComponent("ios")
                    .URLByAppendingPathComponent(bundleIdentifier)
                    .URLByAppendingPathComponent("callback");
            }
            else {
                return undefined;
            }
        },
        enumerable: true,
        configurable: true
    });
    SafariWebAuth.prototype.clearSession = function (federated, callback) {
        var logoutURL = federated
            ? new NSURL({ string: "/v2/logout?federated", relativeToURL: this.url })
            : new NSURL({ string: "/v2/logout", relativeToURL: this.url });
        if (Number(platform_1.device.osVersion.split('.')[0]) >= 11.0 && this.authenticationSession) {
            var returnTo = new NSURLQueryItem({ name: "returnTo", value: (this.redirectURL != null) ? this.redirectURL.absoluteString : undefined });
            var clientId = new NSURLQueryItem({ name: "client_id", value: this.clientId });
            var components = new NSURLComponents({ URL: logoutURL, resolvingAgainstBaseURL: true });
            var queryItems = (components.queryItems != null) ? utils_1.nsArrayToJSArray(components.queryItems) : [];
            components.queryItems = utils_1.jsArrayToNSArray(queryItems.concat([returnTo, clientId]));
            var clearSessionURL = components.URL;
            var redirectURL = returnTo.value;
            if (clearSessionURL == null || redirectURL == null) {
                return callback(false);
            }
            var clearSession = new safariAuthenticationSessionCallback_1.SafariAuthenticationSessionCallback(clearSessionURL, redirectURL, callback);
            this.storage.store(clearSession);
        }
        else {
            var controller = silentSafariViewController_1.SilentSafariViewController.alloc().initWithURLCallback(logoutURL, function (result) { return callback(result); });
            if (this.logger != null) {
                this.logger.trace(logoutURL, "Safari");
            }
            this.presenter.present(controller);
        }
    };
    SafariWebAuth.NoBundleIdentifier = 'com.auth0.this-is-no-bundle';
    return SafariWebAuth;
}(webAuth_1.WebAuth));
exports.SafariWebAuth = SafariWebAuth;
function generateDefaultState() {
    var data = new NSMutableData({ length: 32 });
    var result = SecRandomCopyBytes(kSecRandomDefault, data.length, data.mutableBytes);
    if (result !== 0) {
        return undefined;
    }
    return utils_1.a0_encodeBase64URLSafe(data);
}
//# sourceMappingURL=safariWebAuth.js.map